
#include <stdio.h>
#include <stdlib.h>

#include <NDS/NDS.h>
#include <NDS/ARM9/console.h> //basic print funcionality

#include "imfs.h"
#include "strtool.h"

typedef struct {
  u32 Hash;
  u32 Offset;
} TIMFS_FileInfo;

u32 ImplantFileCount=0;

TIMFS_FileInfo *IMFS_FileInfo=NULL;
u8 *IMFS_FileData=NULL;

void IMFS_Init(void *_data)
{
  ImplantFileCount=0;
  IMFS_FileInfo=NULL;
  IMFS_FileData=NULL;
  
  if(_data==NULL) return;
  
  u8 *data=(u8*)_data;
  
  data+=4*3; // SkipID
  
  ImplantFileCount=*(u32*)data;
  data+=4;
  
  IMFS_FileInfo=(TIMFS_FileInfo*)data;
  IMFS_FileData=(u8*)&IMFS_FileInfo[ImplantFileCount];
  
//  consolePrintf("*FileInfo=0x%08X FileCount=%d\n", (u32)data,ImplantFileCount);
}

u32 IMFS_GetImplantFileCount(void)
{
  return(ImplantFileCount);
}

u32 IMFS_GetStrCRC32Fast(const char *str)
{
  if(str==NULL) return((u32)-1);
  
  u32 CRC=0;
  while(*str!=0){
    CRC=CRC ^ (u32)*str;
    CRC=((CRC >> (32-8)) & 0x000000ff)+((CRC & 0x00ffffff) << 8);
    str++;
  }
  
  return(CRC);
}

u32 IMFS_GetIndexFromFilename(const char *fn)
{
  if(fn==NULL) return((u32)-1);
  
  u32 idx;
  u32 SrcHash;
  
  SrcHash=IMFS_GetStrCRC32Fast(fn);
  
  for(idx=0;idx<ImplantFileCount;idx++){
    if(SrcHash==IMFS_FileInfo[idx].Hash){
      char *FindStr=(char*)&IMFS_FileData[IMFS_FileInfo[idx].Offset+4];
      if(isStrEqual(&fn[0],FindStr)==True){
        return(idx);
      }
    }
  }
  
  return((u32)-1);
}

u32 IMFS_GetFilenameSizeFromIndex(u32 idx)
{
  if(idx==(u32)-1) return((u32)-1);
  
  u8 *FileData=&IMFS_FileData[IMFS_FileInfo[idx].Offset];
  return(*(u32*)&FileData[0]);
}

u32 IMFS_GetFilenameSizeFromFilename(const char *fn)
{
  if(fn==NULL) return((u32)-1);
  
  u32 idx=IMFS_GetIndexFromFilename(fn);
  return(IMFS_GetFilenameSizeFromIndex(idx));
}

char* IMFS_GetFilenameFromIndex(u32 idx)
{
  if(idx==(u32)-1) return(NULL);
  
  u8 *FileData=&IMFS_FileData[IMFS_FileInfo[idx].Offset];
  return((char*)&FileData[4]);
}

char* IMFS_GetFilenameFromFilename(const char *fn)
{
  if(fn==NULL) return(NULL);
  
  u32 idx=IMFS_GetIndexFromFilename(fn);
  return(IMFS_GetFilenameFromIndex(idx));
}

u32 IMFS_GetFileDataSizeFromIndex(u32 idx)
{
  if(idx==(u32)-1) return((u32)-1);
  
  u8 *FileData=&IMFS_FileData[IMFS_FileInfo[idx].Offset];
  return(*(u32*)&FileData[4+IMFS_GetFilenameSizeFromIndex(idx)]);
}

u32 IMFS_GetFileDataSizeFromFilename(const char *fn)
{
  if(fn==NULL) return((u32)-1);
  
  u32 idx=IMFS_GetIndexFromFilename(fn);
  return(IMFS_GetFileDataSizeFromIndex(idx));
}

u8* IMFS_GetFileDataFromIndex(u32 idx)
{
  if(idx==(u32)-1) return(NULL);
  
  u8 *FileData=&IMFS_FileData[IMFS_FileInfo[idx].Offset];
  return((u8*)&FileData[4+IMFS_GetFilenameSizeFromIndex(idx)+4]);
}

u8* IMFS_GetFileDataFromFilename(const char *fn)
{
  if(fn==NULL) return(NULL);
  
  u32 idx=IMFS_GetIndexFromFilename(fn);
  return(IMFS_GetFileDataFromIndex(idx));
}

