
#ifndef	jpeg_H
#define	jpeg_H

#include <NDS/NDS.h>

#include "idct.h"

typedef struct {
  u8 code[256];
  u8 bitcount[256];
  u16 bitimage[256];
  u32 size;
} THuffmanTree;

typedef struct {
  u8 data[64];
  u8 Precision;
} TQuantizeTable;

typedef struct {
  u16 DepthPerSample;
  u16 Height,Width;
  u32 SampleMaxHFact,SampleMaxVFact;
  u32 MCUWidthSize,MCUHeightSize;
  u32 MCUWidthCount,MCUHeightCount;
  u32 MCUCount;
} TFlameInfo;

typedef struct {
  u8 HFact,VFact;
  u8 QuantizeTableIndex;
} TSampleInfo;

typedef struct {
  u32 HuffmanTableDC,HuffmanTableAC;
  THuffmanTree *HuffmanTreeDC;
  THuffmanTree *HuffmanTreeAC;
  TSampleInfo *SampleInfo;
  TQuantizeTable *QuantizeTable;
  s32 DCDiff;
} TScanInfo;

typedef struct {
  u32 PosBit;
  s16 DCDiff[3];
  b8 CheckSegment_EndFlag;
} TMCUCache;

typedef struct {
  u32 Life;
  u16 *BM;
  u32 ypos;
} TLCBM; // LineCacheBitmap

#define LCBM_False (0xffffffff)

TLCBM *LCBM;
u32 LCBMLifeCur;

u32 LCBMCount;
u32 LCBMWidth;
#define LCBMHeight (16)

#define JpegTempBMWidth (4096)
#define JpegTempBMHeight (16)

u16 *JpegTempBM;

typedef s16 TIntBlock;

// jpeg_zigzag_order jutils.c
/*
static u32 zigzag[64]={
   0,  1,  5,  6, 14, 15, 27, 28,
   2,  4,  7, 13, 16, 26, 29, 42,
   3,  8, 12, 17, 25, 30, 41, 43,
   9, 11, 18, 24, 31, 40, 44, 53,
  10, 19, 23, 32, 39, 45, 52, 54,
  20, 22, 33, 38, 46, 51, 55, 60,
  21, 34, 37, 47, 50, 56, 59, 61,
  35, 36, 48, 49, 57, 58, 62, 63};
*/

// jpeg_natural_order jutils.c
const int zigzag[64] = {
  0,  1,  8, 16,  9,  2,  3, 10,
 17, 24, 32, 25, 18, 11,  4,  5,
 12, 19, 26, 33, 40, 48, 41, 34,
 27, 20, 13,  6,  7, 14, 21, 28,
 35, 42, 49, 56, 57, 50, 43, 36,
 29, 22, 15, 23, 30, 37, 44, 51,
 58, 59, 52, 45, 38, 31, 39, 46,
 53, 60, 61, 54, 47, 55, 62, 63};

#define YUVMode_Unknown (0)
#define YUVMode_111 (1)
#define YUVMode_411 (2)
#define YUVMode_100 (3)

u32 BitStreamStartPosition;
u32 YUVMode;
TFlameInfo FlameInfo;
TQuantizeTable QuantizeTable[4];
THuffmanTree HuffmanTree[2][2];
u32 ScanCount;
TScanInfo ScanInfo[3];
u32 SampleCount;
TSampleInfo SampleInfo[3];
u32 DelayRestartInterval;
b8 CheckSegment_EndFlag=False;

TIntBlock Y0Block[64],Y1Block[64],Y2Block[64],Y3Block[64],CbBlock[64],CrBlock[64];

TMCUCache *MCUCache;

// jpeg_decodeimage.c

void CheckSegment(u32 *MCUCnt);
void LoadIntBlock(u32 ScanCnt,TIntBlock *IntBlock);
void IntBlockDraw111(u16 *DrawPtr);
void IntBlockDraw411(u16 *DrawPtr);
void IntBlockDraw100(u16 *DrawPtr);
void (*IntBlockDraw)(u16 *DrawPtr);
void DecodeImageOne(u32 *_MCUCount,u16 *DrawPtr);
void DrawImageOneLine(u32 BaseY,u32 LCBMNum);

// jpeg_fs.c

void rfsInit(u8 *ptr);
void rfsClose(void);
attrinline u8 *rfsReadBuf(u32 size);
attrinline u8 rfs8bit(void);
attrinline u16 rfs16bit(void);

attrinline u32 rfsGetPosition(void);
attrinline void rfsSetPosition(u32 _pos);

// jpeg_marker.c

b8 AnalizeJpegMarker(void);

u32 Marker__GetSize(void);
u32 Marker__GetNextPosition(void);
void Marker_Unknown(void);
void Marker_APP0(void);
void Marker_APPx(void);
void Marker_DQT(void);
void Marker_SOF0(void);
void Marker_DHT(void);
void Marker_DRI(void);
void Marker_SOS(void);

#include "jpeg_fs.h"
#include "jpeg_marker.h"
#include "jpeg_decodeimage.h"

void JpegInit(u32 count,u32 width)
{
  Initialize_Fast_IDCT();
  
  JpegTempBM=(u16*)safemalloc(JpegTempBMWidth*JpegTempBMHeight*2);
  
  width=((width+3)/4)*4;
  
  LCBMCount=count;
  LCBMWidth=width;
  
  LCBMLifeCur=0;
  
  LCBM=(TLCBM*)safemalloc(count*sizeof(TLCBM));
  
  for(u32 idx=0;idx<LCBMCount;idx++){
    LCBM[idx].Life=0;
    LCBM[idx].BM=NULL;
    LCBM[idx].ypos=0;
  }
  
  MCUCache=(TMCUCache*)safemalloc(count*sizeof(TMCUCache));
  
  TMCUCache *_MCUCache=&MCUCache[0];
  _MCUCache->PosBit=0;
  _MCUCache->DCDiff[0]=0;
  _MCUCache->DCDiff[1]=0;
  _MCUCache->DCDiff[2]=0;
  _MCUCache->CheckSegment_EndFlag=False;
}

void JpegFree(void)
{
  free(JpegTempBM); JpegTempBM=NULL;
  
  for(u32 idx=0;idx<LCBMCount;idx++){
    if(LCBM[idx].BM!=NULL){
      free(LCBM[idx].BM); LCBM[idx].BM=NULL;
    }
  }
  
  free(LCBM); LCBM=NULL;
  
  free(MCUCache); MCUCache=NULL;
}

void LoadIWRAMMappedCode(u32 YUVMode)
{
  switch(YUVMode){
    case YUVMode_Unknown:
      consolePrintf("Unknown YUVMode");
      IntBlockDraw=0;
      break;
    case YUVMode_111:
      IntBlockDraw=IntBlockDraw111;
      break;
    case YUVMode_411:
      IntBlockDraw=IntBlockDraw411;
      break;
    case YUVMode_100:
      IntBlockDraw=IntBlockDraw100;
      break;
    default:
      consolePrintf("Illigal YUVMode");
      IntBlockDraw=0;
      while(1);
  }
}

u32 LCBM_FindCache(u32 y)
{
  u32 i;
  
  for(i=0;i<LCBMCount;i++){
    if(LCBM[i].BM!=NULL){
      if(LCBM[i].ypos==y) return(i);
    }
  }
  
  return(LCBM_False);
}

u32 LCBM_FindFree(void)
{
  u32 i;
  
  for(i=0;i<LCBMCount;i++){
    if(LCBM[i].BM==NULL){
      return(i);
    }
  }
  
  return(LCBM_False);
}

u32 LCBM_FindOldCache(void)
{
  u32 i;
  u32 LL=0xffffffff;
  u32 LLnum=0;
  
  for(i=0;i<LCBMCount;i++){
    if(LCBM[i].BM!=NULL){
      if(LL>LCBM[i].Life){
        LL=LCBM[i].Life;
        LLnum=i;
      }
    }
  }
  
  return(LLnum);
}

#endif
