
#include <NDS/NDS.h>
#include <NDS/ARM9/console.h> //basic print funcionality

#include <stdio.h>
#include <stdlib.h>

#define attrinline __attribute__ ((always_inline, const)) static inline 
//#define attrinline 

#define VRAM0 ((u16*)(0x6000000))

#include "_const.h"
#include "boolean.h"
#include "memtool.h"

#include "maindef.h"

#define ScreenWidth (256)
#define ScreenHeight (192)

#include "GBATools.h"
#include "VRAMTool.h"

#include "imfs.h"
#include "IMFSROM.h"

s32 BaseX,BaseY;

#include "jpeg.h"

void mainloop(void);

void Jpeg_mainloop(u32 IMFSIndex);

void LoadIWRAMMappedCode(u32 YUVMode);

b8 tpDragFlag=False;
s32 tpX,tpY;

// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------

s32 FS_TopIndex;
s32 FS_CursorIndex;

void FS_RefreshList(void)
{
  u32 idxcnt=IMFS_GetImplantFileCount();
  
  VRAM_Clear();
  
  for(s32 idx=FS_TopIndex;idx<(s32)idxcnt;idx++){
    s32 y=idx*8;
    
    if((-7<=y)&&(y<ScreenHeight)){
      if(idx==FS_CursorIndex){
        VRAM_WriteChar(0*8,idx*8,'>');
      }
      VRAM_WriteString(1*8,idx*8,IMFS_GetFilenameFromIndex(idx));
    }
  }
  
}

void FS_StartJpegFromIndex(s32 idx)
{
  u32 idxcnt=IMFS_GetImplantFileCount();
  
  if((0<=idx)&&(idx<(s32)idxcnt)){
    Jpeg_mainloop(idx);
    FS_RefreshList();
    while(((~KEYS)&0x3ff)!=0);
  }
}

void FS_StartJpegFromCursor(void)
{
  FS_StartJpegFromIndex(FS_CursorIndex);
}

void FS_SetCursor(s32 idx)
{
  u32 idxcnt=IMFS_GetImplantFileCount();
  
  if(idx<0) idx=0;
  if((s32)idxcnt<=idx) idx=idxcnt-1;
  
  FS_CursorIndex=idx;
}

void FS_UpCursor(void)
{
  FS_SetCursor(FS_CursorIndex-1);
}

void FS_DownCursor(void)
{
  FS_SetCursor(FS_CursorIndex+1);
}

// ------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------

void RefreshScreen(void)
{
  LoadIWRAMMappedCode(YUVMode);
  
  u32 ycnt,ylimit;
  
  ylimit=(ScreenHeight/16)+1;
  
  for(ycnt=0;ycnt<ylimit;ycnt++){
    u16 *drawbm;
    u32 srcypos;
    
    srcypos=(BaseY/16)+ycnt;
    
    u32 cachednum=LCBM_FindCache(srcypos);
    
    if(cachednum!=LCBM_False){
      LCBM[cachednum].Life=LCBMLifeCur; LCBMLifeCur++;
      drawbm=LCBM[cachednum].BM;
      }else{
      u32 freenum=LCBM_FindFree();
      
      if(freenum!=LCBM_False){
        LCBM[freenum].BM=(u16*)safemalloc(LCBMWidth*LCBMHeight*2);
        if(LCBM[freenum].BM==NULL){
          freenum=LCBM_FindOldCache();
        }
        }else{
        freenum=LCBM_FindOldCache();
      }
      
      LCBM[freenum].Life=LCBMLifeCur; LCBMLifeCur++;
      LCBM[freenum].ypos=srcypos;
      DrawImageOneLine(srcypos,freenum);
      drawbm=LCBM[freenum].BM;
    }
    
    u16 *srcbm,*dstbm;
    u32 y;
    
    for(y=0;y<LCBMHeight;y++){
      s32 yofs;
      s32 base;
      
      yofs=0-(BaseY % 16);
      base=(ycnt*LCBMHeight)+y+yofs;
      
      if((0<=base)&&(base<ScreenHeight)){
        srcbm=&drawbm[BaseX+(y*LCBMWidth)];
        dstbm=&VRAMBuf[base*ScreenWidth];
        MemCopy32DMA3(srcbm,dstbm,ScreenWidth*2);
      }
    }
  }
  
}

/*

float per=2.0;

void RefreshScreen(void)
{
  LoadIWRAMMappedCode(YUVMode);
  
  u32 ycnt,ylimit;
  
  ylimit=(ScreenHeight/per/16)+1;
  
  for(ycnt=0;ycnt<ylimit;ycnt++){
    u16 *drawbm;
    u32 srcypos;
    
    srcypos=(BaseY/16)+ycnt;
    
    u32 cachednum=LCBM_FindCache(srcypos);
    
    if(cachednum!=LCBM_False){
      LCBM[cachednum].Life=LCBMLifeCur; LCBMLifeCur++;
      drawbm=LCBM[cachednum].BM;
      }else{
      u32 freenum=LCBM_FindFree();
      
      if(freenum!=LCBM_False){
        LCBM[freenum].BM=(u16*)safemalloc(LCBMWidth*LCBMHeight*2);
        if(LCBM[freenum].BM==NULL){
          freenum=LCBM_FindOldCache();
        }
        }else{
        freenum=LCBM_FindOldCache();
      }
      
      LCBM[freenum].Life=LCBMLifeCur; LCBMLifeCur++;
      LCBM[freenum].ypos=srcypos;
      DrawImageOneLine(srcypos,freenum);
      drawbm=LCBM[freenum].BM;
    }
    
    u16 *srcbm,*dstbm;
    u32 y;
    
    for(y=0;y<LCBMHeight*per;y++){
      s32 yofs;
      s32 base;
      
      yofs=0-(BaseY % 16);
      base=(ycnt*LCBMHeight)+yofs;
      base=base*per;
      base+=y;
      
      if((0<=base)&&(base<ScreenHeight)){
        srcbm=&drawbm[(u32)(BaseX/per)+((u32)(y/per)*LCBMWidth)];
        dstbm=&VRAMBuf[base*ScreenWidth];
        u32 px;
        for(px=0;px<ScreenWidth;px++){
          dstbm[px]=srcbm[(u32)(px/per)];
        }
      }
    }
  }
  
}

*/

int main(void)
{
  // irqs are nice
  irqInitHandler(irqDefaultHandler);
  irqSet(IRQ_VBLANK, 0);
  
  POWER_CR = POWER_ALL_2D;
  
  //set our display modes
  vramSetMainBanks(VRAM_A_MAIN_BG_0x6000000, VRAM_B_MAIN_BG_0x6020000, VRAM_C_SUB_BG, VRAM_D_LCD);
//  videoSetMode(MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE | DISPLAY_SPR_ACTIVE | DISPLAY_SPR_1D | DISPLAY_SPR_1D_BMP);
  videoSetMode(MODE_5_2D | DISPLAY_BG3_ACTIVE );
//  videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE | DISPLAY_SPR_ACTIVE | DISPLAY_SPR_1D | DISPLAY_SPR_1D_BMP);
  videoSetModeSub(MODE_0_2D | DISPLAY_BG3_ACTIVE );
  
  BG3_CR = BG_BMP16_256x256 | BG_BMP_BASE(0) | BG_PRIORITY_0;
  
	//these are rotation backgrounds so you must set the rotation attributes:
    //these are fixed point numbers with the low 8 bits the fractional part
    //this basicaly gives it a 1:1 translation in x and y so you get a nice flat bitmap
        BG3_XDX = 1 << 8;
        BG3_XDY = 0;
        BG3_YDX = 0;
        BG3_YDY = 1 << 8;
    //our bitmap looks a bit better if we center it so scroll down (256 - 192) / 2 
        BG3_CX = 0;
        BG3_CY = 0;
  
  SUB_BG3_CR = BG_32x32 | BG_MAP_BASE(31) | BG_TILE_BASE(0) | BG_PRIORITY_0;
  
  BG_PALETTE_SUB[255] = RGB15(31,31,31);//by default font will be rendered with color 255
  
  //consoleInit() is a lot more flexible but this gets you up and running quick
  consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
  
  consolePrintSet(0,0);
  
  consolePrintf(ROMTITLE);
  consolePrintf("\n");
  consolePrintf(ROMVERSION);
  consolePrintf("\n");
  consolePrintf(ROMDATE);
  consolePrintf("\n");
  
  consolePrintf("\n");
  consolePrintf(IMFSTitle);
  consolePrintf("\n");
  
  consolePrintf("\n");
  consolePrintf("A button=StartDecode\n");
  consolePrintf("B button=ReturnFileList\n");
  consolePrintf("Cursor=Scroll\n");
  consolePrintf("TouchPanel=Scroll\n");
  consolePrintf("\n");
  
  IMFS_Init((u8*)&IMFSROM[0]);
  
  mainloop();
  
  while(1);
}

b8 Jpeg_ProcessKeys(u16 loadkeys)
{
  if(loadkeys==0) return(True);
  
  s32 val=1;
  
  if(loadkeys & KEY_A){
    val*=4;
  }
  if(loadkeys & KEY_B){
    return(False);
  }
  
  if(loadkeys & KEY_LEFT){
    BaseX-=val;
  }
  if(loadkeys & KEY_RIGHT){
    BaseX+=val;
  }
  if(loadkeys & KEY_UP){
    BaseY-=val;
  }
  if(loadkeys & KEY_DOWN){
    BaseY+=val;
  }
  
  if(loadkeys & KEY_SELECT){
    lcdSwap();
  }
  if(loadkeys & KEY_START){
    lcdSwap();
  }
  if((loadkeys&KEY_L)&&(loadkeys&KEY_R)){
    return(False);
  }
  
  return(True);
}

b8 Jpeg_ProcessTouchPad(void)
{
  if((IPC->buttons&IPC_PEN_DOWN)!=0){
    tpDragFlag=False;
    }else{
    s32 Xpx,Ypx;
    
    Xpx=IPC->touchXpx;
    Ypx=IPC->touchYpx;
    
    if(tpDragFlag==False){
      tpDragFlag=True;
      }else{
      if(((Xpx-tpX)!=0)||((Ypx-tpY)!=0)){
        BaseX-=(Xpx-tpX)*4;
        BaseY-=(Ypx-tpY)*4;
      }
    }
    
    tpX=Xpx;
    tpY=Ypx;
  }
  
  return(True);
}

void Jpeg_ShowFlameInfo(void)
{
  consolePrintf("MCUCount (%d*%d)=%d\n",FlameInfo.MCUWidthCount,FlameInfo.MCUHeightCount,FlameInfo.MCUCount);
  consolePrintf("PicSize (%d*%d)=%d\n",FlameInfo.Width,FlameInfo.Height,FlameInfo.Width*FlameInfo.Height);
  
  consolePrintf("MaxFact H=%d F=%d\n",FlameInfo.SampleMaxHFact,FlameInfo.SampleMaxVFact);
  consolePrintf("DRI %d\n",DelayRestartInterval);
  
  consolePrintf("BitStreamStartPosition=%d\n",BitStreamStartPosition);
}

b8 Jpeg_mainstart(u32 IMFSIndex)
{
  consolePrintSet(0,0);
  PrintFreeMem();
  
  VRAM_Clear();
  
  u8 *JpegData=IMFS_GetFileDataFromIndex(IMFSIndex);
  
  consolePrintf("JpegFilename %s\n",IMFS_GetFilenameFromIndex(IMFSIndex));
  consolePrintf("JpegData 0x%08x %dbyte\n",JpegData,IMFS_GetFileDataSizeFromIndex(IMFSIndex));
  
  rfsInit(JpegData);
  rfsStartFlame();
  
  if(AnalizeJpegMarker()==False){
    consolePrintf("JpegHeader AnalizeError.");
    return(False);
  }
  
  Jpeg_ShowFlameInfo();
  
  rbsInit(JpegData+BitStreamStartPosition);
  
  JpegInit(FlameInfo.MCUHeightCount,FlameInfo.Width);
  
  RefreshScreen();
  
  return(True);
}

void Jpeg_mainloop(u32 IMFSIndex)
{
  u16 loadkeys;
  
  BaseX=0;
  BaseY=0;
  
  Jpeg_mainstart(IMFSIndex);
  
  b8 LoopFlag=True;
  
  while(LoopFlag==True){
    
    swiWaitForVBlank();
    
    s32 LastX=BaseX;
    s32 LastY=BaseY;
    
    {
      loadkeys=KEYS_CUR&(~KEY_TOUCH);
      if(LoopFlag==True) LoopFlag=Jpeg_ProcessKeys(loadkeys);
      if(LoopFlag==True) LoopFlag=Jpeg_ProcessTouchPad();
    }
    
    if(BaseX<0) BaseX=0;
    if((FlameInfo.Width-ScreenWidth)<BaseX) BaseX=FlameInfo.Width-ScreenWidth;
    
    if(BaseY<0) BaseY=0;
    if((FlameInfo.Height-ScreenHeight)<BaseY) BaseY=FlameInfo.Height-ScreenHeight;
    
    if((LastX!=BaseX)|(LastY!=BaseY)){
      RefreshScreen();
    }
    
  }
  
  JpegFree();
  
}

// ---------------------------------------------------------------

void ProcessTouchPad(void)
{
  if((IPC->buttons&IPC_PEN_DOWN)!=0){
    tpDragFlag=False;
    }else{
    s32 Xpx,Ypx;
    
    Xpx=IPC->touchXpx;
    Ypx=IPC->touchYpx;
    
    if(tpDragFlag==False){
      tpDragFlag=True;
      FS_SetCursor(FS_TopIndex+(Ypx/8));
      FS_StartJpegFromCursor();
      }else{
      s32 lastx,lasty;
      
      lastx=BaseX;
      lasty=BaseY;
      
      BaseY-=Ypx-tpY;
      if(BaseY<0) BaseY=0;
      BaseX-=Xpx-tpX;
      if(BaseX<0) BaseX=0;
      
      if((lastx!=BaseX)||(lasty!=BaseY)){
      }
    }
    
    tpX=Xpx;
    tpY=Ypx;
  }
  
}

void ProcessKeys(u16 loadkeys)
{
  if(loadkeys==0) return;
  
  if(loadkeys & KEY_UP){
    FS_UpCursor();
    FS_RefreshList();
  }
  if(loadkeys & KEY_DOWN){
    FS_DownCursor();
    FS_RefreshList();
  }
  
  if(loadkeys & KEY_A){
    FS_StartJpegFromCursor();
  }
  
  if(loadkeys & KEY_B){
  }
  if(loadkeys & KEY_SELECT){
  }
  if(loadkeys & KEY_START){
  }
  if((loadkeys&KEY_L)&&(loadkeys&KEY_R)){
    while(1);
  }
  
  while(((~KEYS)&0x3ff)!=0);
}

void mainloop(void)
{
  FS_TopIndex=0;
  FS_CursorIndex=0;
  
  FS_RefreshList();
  
  while(((~KEYS)&0x3ff)!=0);
  
  while(1){
    u16 loadkeys=(~KEYS)&0x3ff;
    
    ProcessKeys(loadkeys);
    
    ProcessTouchPad();
  }
  
  while(1);
}


