
#ifndef memtool_h
#define memtool_h

#include <NDS/NDS.h>
#include <NDS/ARM9/console.h> //basic print funcionality

#include "string.h"

#include "boolean.h"

#define attrinline __attribute__ ((always_inline, const)) static inline 

attrinline void MemCopy16CPU(void *src,void *dst,u32 len)
{
  memmove(dst,src,len);
}

attrinline void MemCopy32CPU(void *src,void *dst,u32 len)
{
  memmove(dst,src,len);
}

attrinline void MemCopy32DownCPU(void *src,void *dst,u32 len)
{
  memmove(dst,src,len);
}

attrinline void MemSet16CPU(u16 v,void *dst,u32 len)
{
  u16 *_dst=(u16*)dst;
  
  len/=2;
  
  for(;len!=0;len--){
    *_dst=v;
    _dst++;
  }
}

attrinline void MemSet32CPU(u32 v,void *dst,u32 len)
{
  u32 *_dst=(u32*)dst;
  
  len/=4;
  
  for(;len!=0;len--){
    *_dst=v;
    _dst++;
  }
}

attrinline void MemCopy16DMA3(void *src,void *dst,u32 len)
{
  DMA_SRC(3) = (uint32)src;
  DMA_DEST(3) = (uint32)dst;
  DMA_CR(3) = DMA_ENABLE | DMA_SRC_INC | DMA_DST_INC | DMA_16_BIT | (len>>1);
  while(DMA_CR(3) & DMA_BUSY);
}

attrinline void MemCopy32DMA3(void *src,void *dst,u32 len)
{
  DMA_SRC(3) = (uint32)src;
  DMA_DEST(3) = (uint32)dst;
  DMA_CR(3) = DMA_ENABLE | DMA_SRC_INC | DMA_DST_INC | DMA_32_BIT | (len>>2);
  while(DMA_CR(3) & DMA_BUSY);
}

attrinline void MemCopy32DownDMA3(void *src,void *dst,u32 len)
{
  MemCopy32DownCPU(src,dst,len);
}

attrinline void MemSet16DMA3(u16 v,void *dst,u32 len)
{
  u32 vdual=(u32)v+((u32)v<<16);
  
  DMA_SRC(3) = (uint32)&vdual;
  DMA_DEST(3) = (uint32)dst;
  DMA_CR(3) = DMA_ENABLE | DMA_SRC_FIX | DMA_DST_INC | DMA_16_BIT | (len>>1);
  while(DMA_CR(3) & DMA_BUSY);
}

attrinline void MemSet32DMA3(u32 v,void *dst,u32 len)
{
  DMA_SRC(3) = (uint32)&v;
  DMA_DEST(3) = (uint32)dst;
  DMA_CR(3) = DMA_ENABLE | DMA_SRC_FIX | DMA_DST_INC | DMA_32_BIT | (len>>2);
  while(DMA_CR(3) & DMA_BUSY);
}

void *safemalloc(int size)
{
  if(size<=0) return(NULL);
  
  void *ptr;
  
  ptr=malloc(size+(64*1024)); // 64kb͕Kc
  
  if(ptr==NULL) return(NULL);
  
  u32 adr=(u32)ptr;
  
  if((adr+size)<0x02000000){ // 擪AhXCO
    free(ptr);
    return(NULL);
  }
  
  if((0x02000000+(4*1024*1024))<=adr){ // I[AhXCゾ
    free(ptr);
    return(NULL);
  }
  
  // 64kb]mۂ蒼
  
  free(ptr);
  ptr=malloc(size);
  
  return(ptr);

}

void PrintFreeMem(void)
{
  s32 i;
  void *p;
  void *FreeMemPtr=NULL;
  u32 FreeMemSize=0;
  
  for(i=1*1024;i<4096*1024;i+=1024){
    p=safemalloc(i+(64*1024));
    if(p==NULL) break;
    FreeMemPtr=p;
    FreeMemSize=i;
    free(p);
  }
  
  consolePrintf("FreeMem=0x%08x %dkb    \n",(u32)FreeMemPtr,FreeMemSize/1024);
}

#endif

